package nkode

import (
	"go-nkode/core/model"
)

type DbAccessor interface {
	GetCustomer(model.CustomerId) (*model.Customer, error)
	GetUser(model.Username, model.CustomerId) (*model.User, error)
	WriteNewCustomer(model.Customer) error
	WriteNewUser(model.User) error
	UpdateUserInterface(model.UserId, model.UserInterface) error
	Renew(model.CustomerId) error
	RefreshUser(model.User, []int, model.CustomerAttributes) error
}
